local api = require "api"
local config = require "config"
local geo = require "geo"
local psl = require "psl"
local utils = require "utils"

local headers = ngx.req.get_headers()

if headers["Jasima-Api-Key"] then
    api.handle_api_request(headers["Jasima-Api-Key"])
end

local jasima_key = config.get_jasima_key()
ngx.ctx.jasima_key = jasima_key
if not jasima_key then
    ngx.log(ngx.DEBUG, "no key specified via header or cookie")
    return ngx.exit(400)
end

local err
ngx.ctx.jasima_config, err = config.load_config(jasima_key)
if err then
    ngx.status = 500
    ngx.log(ngx.ERR, "could not load config: " .. err)
    return ngx.exit(500)
end

if not ngx.ctx.jasima_config then
    ngx.status = 403
    ngx.log(ngx.INFO, "requested a canonical host that has no configuration specified: " .. jasima_host)
    ngx.exit(403)
end

if not ngx.ctx.jasima_config.host_canonical then
    ngx.log(ngx.DEBUG, "no origin host specified in config")
    return ngx.exit(400)
end
ngx.ctx.jasima_host = ngx.ctx.jasima_config.host_canonical

local country = geo.viewer_country()
if not ngx.ctx.jasima_config.geo_redirect_disable then
    ngx.log(ngx.DEBUG, "geo_redirect_disable not configured for site " .. ngx.ctx.jasima_host)
    if not geo.needs_mirror(country) then
        ngx.log(ngx.DEBUG, "mirror is not needed for viewer in " .. country .. " for " .. ngx.ctx.jasima_host .. ", redirecting")
        local request_uri = ngx.var.request_uri
        local new_url = "https://" .. ngx.ctx.jasima_host .. request_uri
        return ngx.redirect(new_url, ngx.HTTP_MOVED_TEMPORARILY)
    end
end

-- Get jasima_pool (not critical)
local jasima_pool = config.get_jasima_pool()
ngx.ctx.jasima_pool_map, err = config.load_pool_mapping(jasima_pool)
if err then
    ngx.status = 500
    ngx.log(ngx.WARN, "could not load requested pool mapping: " .. err)
    return ngx.exit(500)
end

-- Remove the headers that should not be proxied to the origin
for k, v in pairs(headers) do
  if k:lower():match("^jasima%-") then
    ngx.req.clear_header(k)
  end
end

-- Add additional headers that have been specified in the configuration
if ngx.ctx.jasima_config.headers then
    for k, v in pairs(ngx.ctx.jasima_config.headers) do
        ngx.req.set_header(k, v)
    end
end

local host_connect = ngx.ctx.jasima_config.host_connect or ngx.ctx.jasima_host
local host_header = ngx.ctx.jasima_config.host_header or ngx.ctx.jasima_host
local host_ssl = ngx.ctx.jasima_config.host_ssl or ngx.ctx.jasima_host

-- Handle first party Tealium installations
if ngx.ctx.jasima_config.first_party_tealium then
    local tealium = ngx.ctx.jasima_config.first_party_tealium
    if tealium.subdomain and tealium.account then
        local _, _, _, matched = ngx.var.request_uri:match("^/utag/([^/]+)/([^/]+)/([^/]+)/utag(.-)js")
        if matched then
            ngx.log(ngx.DEBUG, "Matched as a Tealium universal tag script " .. tostring(matched))
            host_connect = "tags.tiqcdn.com"
            host_header = "tags.tiqcdn.com"
            host_ssl = "tags.tiqcdn.com"
        end
        ngx.ctx.jasima_host_tealium = tealium.subdomain .. "." .. psl.get_registered_domain(ngx.ctx.jasima_host)
        ngx.log(ngx.DEBUG, "Using first-party Tealium hostname " .. ngx.ctx.jasima_host_tealium)
    end
end

-- Handle first party Adobe Analytics
if ngx.ctx.jasima_config.first_party_adobe then
    local adobe = ngx.ctx.jasima_config.first_party_adobe
    if adobe.subdomain and adobe.account then
        local matched = ngx.var.request_uri:match("^/b/ss/")
        if matched then
            ngx.log(ngx.DEBUG, "Matched as an Adobe Analytics request " .. tostring(matched))
            host_connect = adobe.account .. ".sc.omtrdc.net"
            host_header = host_connect
            host_ssl = host_connect
        end
        ngx.ctx.jasima_host_adobe = adobe.subdomain .. "." .. psl.get_registered_domain(ngx.ctx.jasima_host)
        ngx.log(ngx.DEBUG, "Using first-party Adobe Analytics hostname " .. ngx.ctx.jasima_host_adobe)
    end
end

-- Look up the IP to connect to the origin
local upstream_ips = utils.resolve_origin(host_connect)
ngx.ctx.upstream_ips = utils.filter_bogons(upstream_ips)
if #ngx.ctx.upstream_ips == 0 then
    ngx.log(ngx.ERR, "no A records found")
    return ngx.exit(500)
end

-- Set the nginx host variables
ngx.var.jasima_host = ngx.ctx.jasima_host
ngx.var.jasima_host_connect = host_connect
ngx.var.jasima_host_header = host_header
ngx.var.jasima_host_ssl = host_ssl
